@@ -18,10 +18,14 @@ except NameError:
     from io import TextIOWrapper as file
 
 
-from .constants import PREFORMATTED_TAGS, IMPLICIT_SELF_CLOSING_TAGS, PRESERVE_CONTENTS_TAGS
+from .constants import PREFORMATTED_TAGS, IMPLICIT_SELF_CLOSING_TAGS, PRESERVE_CONTENTS_TAGS, INVISIBLE_ROOT_TAG, INVISIBLE_ROOT_TAG_START, INVISIBLE_ROOT_TAG_END
 from .exceptions import MultipleRootNodeException
 from .Tags import AdvancedTag
 
+from .utils import addStartTag, stripIEConditionals
+
+import codecs
+
 
 class AdvancedHTMLFormatter(HTMLParser):
     '''
@@ -60,16 +64,13 @@ class AdvancedHTMLFormatter(HTMLParser):
 
             @param contents - HTML contents
         '''
-        if self.encoding and self.encoding != sys.getdefaultencoding():
-            if pyver == 2:
-                contents = contents.decode(self.encoding)
-            else:
-                contents = contents.encode().decode(self.encoding)
+        contents = stripIEConditionals(contents)
         try:
             HTMLParser.feed(self, contents)
         except MultipleRootNodeException:
             self.reset()
-            HTMLParser.feed(self, '<xxxblank>' + contents + '</xxxblank>')
+
+            HTMLParser.feed(self, "%s%s" %(addStartTag(contents, INVISIBLE_ROOT_TAG_START), INVISIBLE_ROOT_TAG_END))
 
 
 
@@ -87,18 +88,32 @@ class AdvancedHTMLFormatter(HTMLParser):
         else:
             doctypeStr = ''
 
-        if root.tagName == 'xxxblank': # If we had to add a temp tag, don't include it here.
-            return doctypeStr + ''.join([x.outerHTML for x in root.children]) 
-        return doctypeStr + root.outerHTML
-
+        return doctypeStr + ''.join([elem.outerHTML for elem in self.getRootNodes()])
 
     def getRoot(self):
         '''
             getRoot - returns the root Tag 
-                @return - AdvancedTag at root. If you provided multiple root nodes, this will be a "holder" with tagName of 'xxxblank'
+                @return - AdvancedTag at root. If you provided multiple root nodes, this will be a "holder" with tagName value as constants.INVISIBLE_ROOT_TAG
         '''
         return self.root
 
+    def getRootNodes(self):
+        '''
+            getRootNodes - Gets all objects at the "root" (first level; no parent). Use this if you may have multiple roots (not children of <html>)
+                Use this method to get objects, for example, in an AJAX request where <html> may not be your root.
+
+                Note: If there are multiple root nodes (i.e. no <html> at the top), getRoot will return a special tag. This function automatically
+                  handles that, and returns all root nodes.
+
+                @return list<AdvancedTag> - A list of AdvancedTags which are at the root level of the tree.
+        '''
+        root = self.root
+        if not root:
+            return []
+        if root.tagName == INVISIBLE_ROOT_TAG:
+            return list(root.children)
+        return [root]
+
     def setRoot(self, root):
         '''
             setRoot - Sets the root node, and reprocesses the indexes
@@ -148,7 +163,7 @@ class AdvancedHTMLFormatter(HTMLParser):
 
         if isSelfClosing is False:
             self.inTag.append(newTag)
-            if tagName != 'xxxblank':
+            if tagName != INVISIBLE_ROOT_TAG:
                 self.currentIndentLevel += 1
 
 
@@ -164,6 +179,16 @@ class AdvancedHTMLFormatter(HTMLParser):
         '''
         try:
             # Handle closing tags which should have been closed but weren't
+            foundIt = False
+            for i in range(len(self.inTag)):
+                if self.inTag[i].tagName == tagName:
+                    foundIt = True
+                    break
+
+            if not foundIt:
+                sys.stderr.write('WARNING: found close tag with no matching start.\n')
+                return
+                
             while self.inTag[-1].tagName != tagName:
                 oldTag = self.inTag.pop()
                 if oldTag.tagName in PREFORMATTED_TAGS:
@@ -172,7 +197,7 @@ class AdvancedHTMLFormatter(HTMLParser):
                 self.currentIndentLevel -= 1
 
             self.inTag.pop()
-            if tagName != 'xxxblank':
+            if tagName != INVISIBLE_ROOT_TAG:
                 self.currentIndentLevel -= 1
             if tagName in PREFORMATTED_TAGS:
                 self.inPreformatted -= 1
@@ -237,7 +262,7 @@ class AdvancedHTMLFormatter(HTMLParser):
         if isinstance(filename, file):
             contents = filename.read()
         else:
-            with open(filename, 'r') as f:
+            with codecs.open(filename, 'r', encoding=self.encoding) as f:
                 contents = f.read()
         self.feed(contents)
 
