@@ -22,10 +22,14 @@ except NameError:
 
 from collections import defaultdict
 
-from .constants import IMPLICIT_SELF_CLOSING_TAGS, INVISIBLE_ROOT_TAG
+from .constants import IMPLICIT_SELF_CLOSING_TAGS, INVISIBLE_ROOT_TAG, INVISIBLE_ROOT_TAG_START, INVISIBLE_ROOT_TAG_END
 from .exceptions import MultipleRootNodeException
 from .Tags import AdvancedTag, TagCollection
 
+import codecs
+
+from .utils import stripIEConditionals, addStartTag
+
 class AdvancedHTMLParser(HTMLParser):
     '''
         AdvancedHTMLParser - This class parses and allows searching of  documents
@@ -108,6 +112,14 @@ class AdvancedHTMLParser(HTMLParser):
             Internal for parsing
         '''
         try:
+            foundIt = False
+            for i in range(len(self.inTag)):
+                if self.inTag[i].tagName == tagName:
+                    foundIt = True
+                    break
+
+            if not foundIt:
+                return
             # Handle closing tags which should have been closed but weren't
             while self.inTag[-1].tagName != tagName:
                 self.inTag.pop()
@@ -381,16 +393,12 @@ class AdvancedHTMLParser(HTMLParser):
 
             @param contents - Contents
         '''
-        if self.encoding and self.encoding != sys.getdefaultencoding():
-            if pyver == 2:
-                contents = contents.decode(self.encoding)
-            else:
-                contents = contents.encode().decode(self.encoding)
+        contents = stripIEConditionals(contents)
         try:
             HTMLParser.feed(self, contents)
         except MultipleRootNodeException:
             self.reset()
-            HTMLParser.feed(self, '<%s>%s</%s>' %(INVISIBLE_ROOT_TAG, contents, INVISIBLE_ROOT_TAG))
+            HTMLParser.feed(self, "%s%s" %(addStartTag(contents, INVISIBLE_ROOT_TAG_START), INVISIBLE_ROOT_TAG_END))
 
     def parseFile(self, filename):
         '''
@@ -403,7 +411,7 @@ class AdvancedHTMLParser(HTMLParser):
         if isinstance(filename, file):
             contents = filename.read()
         else:
-            with open(filename, 'r') as f:
+            with codecs.open(filename, 'r', encoding=self.encoding) as f:
                 contents = f.read()
         self.feed(contents)
 
